import { Draft, Immutable } from "immer";
import { useLayoutEffect } from "react";
import { ParametricSelector } from "reselect";
import { PersistenceStoragePlugin } from "./plugins/persistenceStorage";
export declare type Selector<TState, TProps = any, TResult = unknown> = ParametricSelector<TState, TProps, TResult>;
export declare const useIsomorphicLayoutEffect: typeof useLayoutEffect;
export declare function toAnonFunction(arg: unknown): () => typeof arg;
export declare const emptyArray: [];
export declare const useUpdate: (persistencePlugin?: PersistenceStoragePlugin<any> | null | undefined) => () => void;
export declare const isClientSide: boolean;
declare type NN<T> = NonNullable<T>;
export declare type IProduce<TStore> = (draft?: (draft: Draft<TStore>) => void | TStore) => Immutable<TStore>;
export declare type IAsyncProduce<TStore> = (draft?: (draft: Draft<TStore>) => Promise<void>) => Promise<Immutable<TStore>>;
export declare type IHooks<TStore> = Record<string, Selector<Immutable<TStore>>>;
export declare type IActions<TStore> = Record<string, (...args: any[]) => (draft: Draft<TStore>) => void | TStore>;
export declare type IAsyncActions<TStore> = Record<string, (produce: IProduce<TStore>) => (...args: any[]) => Promise<void>>;
export declare type IHooksObj<TStore, A extends {
    hooks?: IHooks<TStore>;
} | undefined> = {
    [HookName in keyof NN<NN<A>["hooks"]>]: (props: Parameters<NN<NN<A>["hooks"]>[HookName]>[1] extends undefined ? void : Parameters<NN<NN<A>["hooks"]>[HookName]>[1] | (() => Parameters<NN<NN<A>["hooks"]>[HookName]>[1]), propsDeps?: unknown[]) => ReturnType<NN<NN<A>["hooks"]>[HookName]>;
};
export declare type IActionsObj<TStore, A extends {
    actions?: IActions<TStore>;
} | undefined> = {
    [ActionName in keyof NN<NN<A>["actions"]>]: (...args: Parameters<NN<NN<A>["actions"]>[ActionName]>) => Immutable<TStore>;
};
export declare type IAsyncActionsObj<TStore, A extends {
    asyncActions?: IAsyncActions<TStore>;
} | undefined> = {
    [ActionName in keyof NN<NN<A>["asyncActions"]>]: (...args: Parameters<ReturnType<NN<NN<A>["asyncActions"]>[ActionName]>>) => Promise<Immutable<TStore>>;
};
export declare type IUseStore<TStore> = () => Immutable<TStore>;
export declare type IUseProduce<TStore> = () => {
    asyncProduce: IAsyncProduce<TStore>;
    produce: IProduce<TStore>;
};
export declare type IPersistenceMethod = {
    setItem(key: string, data: any): any;
    getItem(key: string): string | null;
};
export {};
